from django.contrib import admin
from django_receipts.models import Receipt

class ReceiptAdmin(admin.ModelAdmin):
    list_display = ('created', 'checked', 'valid', 'allow')
    list_filter = ('valid', 'allow')

admin.site.register(Receipt, ReceiptAdmin)



from django.db import models

from django_receipts import constants


class Receipt(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    checked = models.DateTimeField(default=None, blank=True, null=True)
    hash = models.CharField(max_length=255, unique=True)
    valid = models.IntegerField(choices=constants.VALID,
                                default=constants.UNCHECKED)
    allow = models.BooleanField(default=True)

    class Meta:
        ordering = ('-created',)
        db_table = 'receipts'

    def unicode(self):
        return self.hash

    def get_status(self):
        if not self.allow:
            return 'invalid'
        return constants.VALID_INVERTED[self.valid]



UNCHECKED = 0
OK = 1
REFUNDED = 2
EXPIRED = 3
PENDING = 4
ERROR = 5

VALID_LOOKUP = {
    'unchecked': UNCHECKED,
    'ok': OK,
    'refunded':  REFUNDED,
    'pending': PENDING,
    'expired': EXPIRED,
    'error': ERROR
}

VALID_INVERTED = dict([(v, k) for k, v in VALID_LOOKUP.iteritems()])
VALID = [(k, v) for k, v in VALID_LOOKUP.iteritems()]



from datetime import datetime
import logging

from receipts.receipts import Receipt
from django_receipts import constants

log = logging.getLogger(__name__)


def check(model, data):
    """
    Checks the validity of the receipt against the verification service.

    `model`: the mode, of the receipt object to update with the results
    `data`: the actual receipt string sent from the client
    """
    model.checked = datetime.now()
    receipt = Receipt(data)
    log.info('Checking receipt: %s' % model.pk)
    try:
        result = receipt.verify_server()['status']
    except:
        log.error('There was an error with the verification.', exc_info=True)
        result = 'error'

    status = constants.VALID_LOOKUP[result]
    model.valid = status
    model.save()
    return constants.VALID_INVERTED[status]



from setuptools import setup, find_packages

setup(
    name='django-receipts',
    version='0.1',
    description='Django app for verifying web app receipts',
    long_description=open('README.rst').read(),
    author='Andy McKay',
    author_email='andym@mozilla.com',
    license='BSD',
    install_requires=['receipts'],
    py_modules=['django_receipts'],
    url='https://github.com/andymckay/django-receipts',
    packages=find_packages(exclude=['ez_setup']),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        ],
    )



from datetime import datetime, timedelta
import hashlib
import json

from django import http
from django.conf import settings
from django.views.decorators.csrf import csrf_exempt
from django.utils.timezone import now

from django_receipts.models import Receipt
from django_receipts.tasks import check

import logging

log = logging.getLogger(__name__)
CHECK_INTERVAL = getattr(settings, 'RECEIPT_CHECK_INTERVAL', 60 * 60)


@csrf_exempt
def receive(request):
    """
    Receives the receipt from the client and maybe fires off a check of that
    receipt if needed.
    """
    if request.method != 'POST':
        return http.HttpResponseNotAllowed(['POST'])

    receipt = request.raw_post_data
    if not receipt:
        return http.HttpResponseBadRequest()

    hash_ = hashlib.md5(receipt).hexdigest()
    log.info('Receipt received: %s, %s' % (hash_, request.META['REMOTE_ADDR']))

    obj, created = Receipt.objects.get_or_create(hash=hash_)
    expired = (not obj.checked or
               (obj.checked + timedelta(seconds=CHECK_INTERVAL)) < now())

    if obj.allow and expired:
        result = check(obj, receipt)
    else:
        result = obj.get_status()

    return http.HttpResponse(json.dumps({'status': result}),
                             content_type='application/json')



from datetime import timedelta
import json

from django.utils.timezone import now
from django.test import TestCase
from django.core.urlresolvers import reverse

import mock

from django_receipts import constants
from django_receipts.models import Receipt
from django_receipts.tasks import check
from receipts.receipts import VerificationError


class TestView(TestCase):
    hash_ = 'acbd18db4cc2f85cedef654fccc4a4d8'  # hash of foo.

    def setUp(self):
        self.url = reverse('receipts.receive')

    def test_get(self):
        assert self.client.get(self.url).status_code == 405

    def test_post_empty(self):
        assert self.client.post(self.url, '',
                                content_type='text/plain').status_code == 400

    @mock.patch('django_receipts.views.check')
    def test_post_valid(self, check):
        check.return_value = 'ok'
        res = self.client.post(self.url, 'foo', content_type='text/plain')
        assert res.status_code == 200
        assert res['Content-Type'] == 'application/json'
        assert json.loads(res.content)['status'] == 'ok'

    @mock.patch('django_receipts.views.check')
    def test_post_invalid(self, check):
        check.return_value = 'invalid'
        res = self.client.post(self.url, 'foo', content_type='text/plain')
        assert json.loads(res.content)['status'] == 'invalid'

    @mock.patch('django_receipts.views.check')
    def test_post_obj(self, check):
        check.return_value = 'ok'
        self.client.post(self.url, 'foo', content_type='text/plain')
        receipts = Receipt.objects.all()
        assert receipts.count() == 1
        assert receipts[0].hash == self.hash_

    @mock.patch('django_receipts.views.check')
    def test_check_called(self, check):
        check.return_value = 'ok'
        Receipt.objects.create(hash=self.hash_)
        self.client.post(self.url, 'foo', content_type='text/plain')
        receipts = Receipt.objects.all()
        assert receipts.count() == 1
        assert check.called

    @mock.patch('django_receipts.views.check')
    def test_check_not_called(self, check):
        check.return_value = 'ok'
        Receipt.objects.create(hash=self.hash_, checked=now())
        self.client.post(self.url, 'foo', content_type='text/plain')
        assert not check.called

    @mock.patch('django_receipts.views.check')
    def test_check_expired(self, check):
        check.return_value = 'ok'
        Receipt.objects.create(hash=self.hash_, checked=now() - timedelta(days=5))
        self.client.post(self.url, 'foo', content_type='text/plain')
        assert check.called

    @mock.patch('django_receipts.views.check')
    def test_blocked(self, check):
        Receipt.objects.create(hash=self.hash_, checked=now(), allow=False,
                               valid=constants.OK)
        res = self.client.post(self.url, 'foo', content_type='text/plain')
        assert json.loads(res.content)['status'] == 'invalid'


@mock.patch('django_receipts.tasks.Receipt.verify_server')
class TestTask(TestCase):

    def setUp(self):
        self.receipt = Receipt.objects.create(hash='abc')

    def test_ok(self, verify):
        verify.return_value = {'status': 'ok'}
        check(self.receipt, 'abc')
        assert self.receipt.get_status() == 'ok'

    def test_error(self, verify):
        verify.side_effect = VerificationError
        check(self.receipt, 'abc')
        assert self.receipt.get_status() == 'error'

    def test_data_error(self, verify):
        verify.return_value = {'?': '?'}
        check(self.receipt, 'abc')
        assert self.receipt.get_status() == 'error'

